@@ -1,7 +1,7 @@
 #!/usr/bin/perl -w
 
 # mailgraph -- an rrdtool frontend for mail statistics
-# copyright (c) 2000-2004 David Schweikert <dws@ee.ethz.ch>
+# copyright (c) 2000-2005 David Schweikert <dws@ee.ethz.ch>
 # released under the GNU General Public License
 
 ######## Parse::Syslog 1.04 (automatically embedded) ########
@@ -138,7 +138,7 @@ sub _next_syslog($)
             (\d+):(\d+):(\d+) # time  -- 3, 4, 5
             (?:\s<\w+\.\w+>)? # FreeBSD's verbose-mode
             \s
-            ([-\w\.]+)        # host  -- 6
+            ([-\w\.\@:]+)     # host  -- 6
             \s+
             (.*)              # text  -- 7
             $/x or do
@@ -291,7 +291,7 @@ use File::Tail;
 use Getopt::Long;
 use POSIX 'setsid';
 
-my $VERSION = "1.10";
+my $VERSION = "1.12";
 
 # config
 my $rrdstep = 60;
@@ -346,6 +346,7 @@ sub usage
 	print "  --only-virus-rrd   update only the virus rrd\n";
 	print "  --rrd-name=NAME    use NAME.rrd and NAME_virus.rrd for the rrd files\n";
 	print "  --rbl-is-spam      count rbl rejects as spam\n";
+	print "  --virbl-is-virus   count virbl rejects as viruses\n";
 
 	exit;
 }
@@ -358,7 +359,7 @@ sub main
 		'daemon_pid|daemon-pid=s', 'daemon_rrd|daemon-rrd=s',
 		'daemon_log|daemon-log=s', 'ignore-localhost!', 'ignore-host=s',
 		'only-mail-rrd', 'only-virus-rrd', 'rrd_name|rrd-name=s',
-		'rbl-is-spam'
+		'rbl-is-spam', 'virbl-is-virus'
 		) or exit(1);
 	usage if $opt{help};
 
@@ -372,6 +373,12 @@ sub main
 	$daemon_rrd_dir = $opt{daemon_rrd} if defined $opt{daemon_rrd};
 	$rrd		= $opt{rrd_name}.".rrd" if defined $opt{rrd_name};
 	$rrd_virus	= $opt{rrd_name}."_virus.rrd" if defined $opt{rrd_name};
+
+	if($opt{daemon} or $opt{daemon_rrd}) {
+		chdir $daemon_rrd_dir or die "mailgraph: can't chdir to $daemon_rrd_dir: $!";
+		-w $daemon_rrd_dir or die "mailgraph: can't write to $daemon_rrd_dir\n";
+	}
+
 	daemonize if $opt{daemon};
 
 	my $logfile = defined $opt{logfile} ? $opt{logfile} : '/var/log/syslog';
@@ -400,8 +407,6 @@ sub main
 
 sub daemonize()
 {
-	chdir $daemon_rrd_dir or die "mailgraph: can't chdir to $daemon_rrd_dir: $!";
-	-w $daemon_rrd_dir or die "mailgraph: can't write to $daemon_rrd_dir\n";
 	open STDIN, '/dev/null' or die "mailgraph: can't read /dev/null: $!";
 	if($opt{verbose}) {
 		open STDOUT, ">>$daemon_logfile"
@@ -437,7 +442,7 @@ sub init_rrd($)
 	my $year_steps = $month_steps*12;
 
 	# mail rrd
- 	if(! -f $rrd and ! $opt{'only-virus-rrd'}) {
+	if(! -f $rrd and ! $opt{'only-virus-rrd'}) {
 		RRDs::create($rrd, '--start', $m, '--step', $rrdstep,
 				'DS:sent:ABSOLUTE:'.($rrdstep*2).':0:U',
 				'DS:recv:ABSOLUTE:'.($rrdstep*2).':0:U',
@@ -515,13 +520,21 @@ sub process_line($)
 					$client =~ /$opt{'ignore-host'}/oi;
 				event($time, 'received');
 			}
-			elsif($opt{'rbl-is-spam'} and $text =~ /^(?:[0-9A-F]+: |NOQUEUE: )?reject: .*: 554.* blocked using/) {
+			elsif($opt{'virbl-is-virus'} and $text =~ /^(?:[0-9A-F]+: |NOQUEUE: )?reject: .*: 554.* blocked using irbl.dnsbl.bit.nl/) {
+				event($time, 'virus');
+			}
+			elsif($opt{'rbl-is-spam'} and $text    =~ /^(?:[0-9A-F]+: |NOQUEUE: )?reject: .*: 554.* blocked using/) {
 				event($time, 'spam');
 			}
 			elsif($text =~ /^(?:[0-9A-F]+: |NOQUEUE: )?reject: /) {
 				event($time, 'rejected');
 			}
 		}
+		elsif($prog eq 'error') {
+			if($text =~ /\bstatus=bounced\b/) {
+				event($time, 'bounced');
+			}
+		}
 		elsif($prog eq 'cleanup') {
 			if($text =~ /^[0-9A-F]+: (?:reject|discard): /) {
 				event($time, 'rejected');
@@ -532,9 +545,15 @@ sub process_line($)
 		if($text =~ /\bmailer=local\b/ ) {
 			event($time, 'received');
 		}
+                elsif($text =~ /\bmailer=relay\b/) {
+                        event($time, 'received');
+                }
 		elsif($text =~ /\bstat=Sent\b/ ) {
 			event($time, 'sent');
 		}
+                elsif($text =~ /\bmailer=esmtp\b/ ) {
+                        event($time, 'sent');
+                }
 		elsif($text =~ /\bruleset=check_XS4ALL\b/ ) {
 			event($time, 'rejected');
 		}
@@ -544,8 +563,13 @@ sub process_line($)
 		elsif($text =~ /\bruleset=check_rcpt\b/ ) {
 			event($time, 'rejected');
 		}
-		elsif($text =~ /\bruleset=check_rcpt\b/ ) {
-			if ($opt{'rbl-is-spam'}) {
+                elsif($text =~ /\bstat=virus\b/ ) {
+                        event($time, 'virus');
+                }
+		elsif($text =~ /\bruleset=check_relay\b/ ) {
+			if (($opt{'virbl-is-virus'}) and ($text =~ /\bivirbl\b/ )) {
+				event($time, 'virus');
+			} elsif ($opt{'rbl-is-spam'}) {
 				event($time, 'spam');
 			} else {
 				event($time, 'rejected');
@@ -558,13 +582,16 @@ sub process_line($)
 			event($time, 'rejected');
 		}
 		elsif($text =~ /\brecipient denied\b/ ) {
-			event($time, 'reject');
+			event($time, 'rejected');
 		}
 		elsif($text =~ /\brecipient unknown\b/ ) {
-			event($time, 'reject');
+			event($time, 'rejected');
 		}
 		elsif($text =~ /\bUser unknown$/i ) {
-			event($time, 'bounce');
+			event($time, 'bounced');
+		}
+		elsif($text =~ /\bMilter:.*\breject=55/ ) {
+			event($time, 'rejected');
 		}
 	}
 	elsif($prog eq 'amavis' || $prog eq 'amavisd') {
@@ -581,11 +608,15 @@ sub process_line($)
 		#	}
 		#}
 		elsif($text =~ /^\([0-9-]+\) (Passed |Blocked )?INFECTED\b/) {
-			event($time, 'virus');# Passed|Blocked inserted since 2004xxxx
+			if($text !~ /\btag2=/) { # ignore new per-recipient log entry (2.2.0)
+				event($time, 'virus');# Passed|Blocked inserted since 2004xxxx
+			}
+		}
+		elsif($text =~ /^\([0-9-]+\) (Passed |Blocked )?BANNED\b/) {
+			if($text !~ /\btag2=/) {
+			       event($time, 'virus');
+			}
 		}
-#		elsif($text =~ /^\([0-9-]+\) (Passed |Blocked )?BANNED\b/) {
-#		       event($time, 'banned');# Passed|Blocked inserted since 2004xxxx
-#		}
 #		elsif($text =~ /^\([0-9-]+\) Passed|Blocked BAD-HEADER\b/) {
 #		       event($time, 'badh');
 #		}
@@ -594,11 +625,21 @@ sub process_line($)
 		}
 	}
 	elsif($prog eq 'vagatefwd') {
-		# Vexira antivirus
+		# Vexira antivirus (old)
 		if($text =~ /^VIRUS/) {
 			event($time, 'virus');
 		}
 	}
+	elsif($prog eq 'hook') {
+		# Vexira antivirus
+		if($text =~ /^\*+ Virus\b/) {
+			event($time, 'virus');
+		}
+		# Vexira antispam
+		if($text =~ /\bcontains spam\b/) {
+			event($time, 'spam');
+		}
+	}
 	elsif($prog eq 'avgatefwd') {
 		# AntiVir MailGate
 		if($text =~ /^Alert!/) {
@@ -615,7 +656,7 @@ sub process_line($)
 		}
 	}
 	elsif($prog eq 'spamd') {
-		if($text =~ /^identified spam/) {
+		if($text =~ /^(?:spamd: )?identified spam/) {
 			event($time, 'spam');
 		}
 	}
@@ -623,7 +664,7 @@ sub process_line($)
 		if($text =~ /spam detected from/) {
 			event($time, 'spam');
 		}
-        }
+	}
 	elsif($prog eq 'spamproxyd') {
 		if($text =~ /^\s*SPAM/ or $text =~ /^identified spam/) {
 			event($time, 'spam');
@@ -656,11 +697,40 @@ sub process_line($)
 			event($time, 'bounced');
 		}
 	}
+	elsif($prog eq 'clamsmtpd') {
+		if($text =~ /status=VIRUS/) {
+			event($time, 'virus');
+		}
+	}
 	elsif($prog eq 'clamav-milter') {
 		if($text =~ /Intercepted/) {
 			event($time, 'virus');
 		}
 	}
+	elsif ($prog eq 'smtp-vilter') {
+		if ($text =~ /clamd: found/) {
+			event($time, 'virus');
+		}
+	}
+	elsif($prog eq 'avmilter') {
+		# AntiVir Milter
+		if($text =~ /^Alert!/) {
+			event($time, 'virus');
+		}
+		elsif($text =~ /blocked\.$/) {
+			event($time, 'virus');
+		}
+	}
+	elsif($prog eq 'bogofilter') {
+		if($text =~ /Spam/) {
+			event($time, 'spam');
+		}
+	}
+	elsif($prog eq 'filter-module') {
+		if($text =~ /\bspam_status\=yes/) {
+			event($time, 'spam');
+		}
+	}
 }
 
 sub event($$)
@@ -731,15 +801,32 @@ B<mailgraph> [I<options>...]
  --only-virus-rrd   update only the virus rrd
  --rrd-name=NAME    use NAME.rrd and NAME_virus.rrd for the rrd files
  --rbl-is-spam      count rbl rejects as spam
+ --virbl-is-virus   count virbl rejects as viruses
 
 =head1 DESCRIPTION
 
 This script does parse syslog and updates the RRD database (mailgraph.rrd) in
 the current directory.
 
+=head2 Log-Types
+
+The following types can be given to --logtype:
+
+=over 10
+
+=item syslog
+
+Traditional "syslog" (default)
+
+=item metalog
+
+Metalog (see http://metalog.sourceforge.net/)
+
+=back
+
 =head1 COPYRIGHT
 
-Copyright (c) 2004 by ETH Zurich. All rights reserved.
+Copyright (c) 2004-2005 by ETH Zurich. All rights reserved.
 
 =head1 LICENSE
 
@@ -768,13 +855,4 @@ S<David Schweikert E<lt>dws@ee.ethz.chE<gt>>
 
 =cut
 
-# Emacs Configuration
-#
-# Local Variables:
-# mode: cperl
-# eval: (cperl-set-style "PerlStyle")
-# mode: flyspell
-# mode: flyspell-prog
-# End:
-#
-# vi: sw=4 et
+# vi: sw=8
